using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace Demo_18_CustomControl_Clock
{
	/// <summary>
	/// Summary description for ClockControl.
	/// </summary>
	public class ClockControl : System.Windows.Forms.Control
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		private int mHour;
		private int mMinute;

		[Category("Behavior"), Description("Specifies the hour.")]
		public int Hour
		{
			get
			{
				return mHour;
			}

			set
			{
				mHour = value;
				this.Invalidate();
			}
		}

		[Category("Behavior"), Description("Specifies the minutes.")]
		public int Minute
		{
			get
			{
				return mMinute;
			}

			set
			{
				mMinute = value;
				this.Invalidate();
			}
		}

		private const int DEFAULT_SIZE = 100;

		public ClockControl()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			this.Size = new Size(DEFAULT_SIZE, DEFAULT_SIZE);
			mHour = DateTime.Now.Hour;
			mMinute = DateTime.Now.Minute;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if( components != null )
					components.Dispose();
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion

		protected override void OnPaint(PaintEventArgs pe)
		{
			Graphics g = pe.Graphics;

			// Draw the circle
			Pen pen = new Pen(Color.Blue, 1);
			g.DrawEllipse(pen, 0, 0, this.Width-1, this.Height-1);
			pen.Dispose();

			// Draw the minute finger
			double minuteFingerAngle =
				(mMinute % 60) * (2*Math.PI/60);
			int minuteFingerLen = this.Width * 45 / 100;
			int x1 = this.Width / 2;
			int y1 = this.Height / 2;
			int x2 = (int) (x1 + minuteFingerLen*Math.Sin(minuteFingerAngle));
			int y2 = (int) (y1 - minuteFingerLen*Math.Cos(minuteFingerAngle));
			pen = new Pen(Color.Red, 2);
			g.DrawLine(pen, x1, y1, x2, y2);
			pen.Dispose();

			// Draw the hour finger
			double hourFingerAngle = 
				(mHour % 12) * (2*Math.PI/12) + 
				(mMinute % 60) * (2*Math.PI/(60*12));
			int hourFingerLen = this.Width * 25 / 100;
			x1 = this.Width / 2;
			y1 = this.Height / 2;
			x2 = (int) (x1 + hourFingerLen*Math.Sin(hourFingerAngle));
			y2 = (int) (y1 - hourFingerLen*Math.Cos(hourFingerAngle));
			pen = new Pen(Color.Yellow, 3);
			g.DrawLine(pen, x1, y1, x2, y2);
			pen.Dispose();

			// Calling the base class OnPaint
			base.OnPaint(pe);
		}

		protected override void OnResize(System.EventArgs e)
		{
			this.Height = this.Width;
			this.Invalidate();
		}
	}
}