using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Threading;

namespace Demo_20_Asynchonous_UI_Update
{
	public class MainForm : System.Windows.Forms.Form
	{
		internal System.Windows.Forms.TextBox TextBoxLastPrimeNumber;
		private System.Windows.Forms.Label LabelLastFoundPrimeNumber;
		private System.Windows.Forms.Button ButtonStart;
		private System.Windows.Forms.Button ButtonStop;

		private Thread mPrimeNumbersFinderThread = null;

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public MainForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.TextBoxLastPrimeNumber = new System.Windows.Forms.TextBox();
			this.LabelLastFoundPrimeNumber = new System.Windows.Forms.Label();
			this.ButtonStart = new System.Windows.Forms.Button();
			this.ButtonStop = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// TextBoxLastPrimeNumber
			// 
			this.TextBoxLastPrimeNumber.Location = new System.Drawing.Point(24, 40);
			this.TextBoxLastPrimeNumber.Name = "TextBoxLastPrimeNumber";
			this.TextBoxLastPrimeNumber.Size = new System.Drawing.Size(176, 22);
			this.TextBoxLastPrimeNumber.TabIndex = 0;
			this.TextBoxLastPrimeNumber.Text = "";
			// 
			// LabelLastFoundPrimeNumber
			// 
			this.LabelLastFoundPrimeNumber.AutoSize = true;
			this.LabelLastFoundPrimeNumber.Location = new System.Drawing.Point(24, 16);
			this.LabelLastFoundPrimeNumber.Name = "LabelLastFoundPrimeNumber";
			this.LabelLastFoundPrimeNumber.Size = new System.Drawing.Size(157, 18);
			this.LabelLastFoundPrimeNumber.TabIndex = 1;
			this.LabelLastFoundPrimeNumber.Text = "Last found prime number:";
			// 
			// ButtonStart
			// 
			this.ButtonStart.Location = new System.Drawing.Point(24, 72);
			this.ButtonStart.Name = "ButtonStart";
			this.ButtonStart.TabIndex = 2;
			this.ButtonStart.Text = "Start";
			this.ButtonStart.Click += new System.EventHandler(this.ButtonStart_Click);
			// 
			// ButtonStop
			// 
			this.ButtonStop.Enabled = false;
			this.ButtonStop.Location = new System.Drawing.Point(128, 72);
			this.ButtonStop.Name = "ButtonStop";
			this.ButtonStop.TabIndex = 3;
			this.ButtonStop.Text = "Stop";
			this.ButtonStop.Click += new System.EventHandler(this.ButtonStop_Click);
			// 
			// MainForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(6, 15);
			this.ClientSize = new System.Drawing.Size(232, 117);
			this.Controls.Add(this.ButtonStop);
			this.Controls.Add(this.ButtonStart);
			this.Controls.Add(this.LabelLastFoundPrimeNumber);
			this.Controls.Add(this.TextBoxLastPrimeNumber);
			this.Font = new System.Drawing.Font("Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(204)));
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedDialog;
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "MainForm";
			this.Text = "Asynchronous UI Update Demo";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.MainForm_Closing);
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new MainForm());
		}

		internal void ShowPrimeNumber(long aNumber)
		{
			TextBoxLastPrimeNumber.Text = aNumber.ToString();
		}

		private void ButtonStart_Click(object sender, System.EventArgs e)
		{
			ButtonStart.Enabled = false;
			ButtonStop.Enabled = true;
			PrimeNumbersFinder finder = new PrimeNumbersFinder(this);
			mPrimeNumbersFinderThread = 
				new Thread(new ThreadStart(finder.FindPrimeNumbers));
			mPrimeNumbersFinderThread.Start();
		}

		private void ButtonStop_Click(object sender, System.EventArgs e)
		{
			ButtonStart.Enabled = true;
			ButtonStop.Enabled = false;
			mPrimeNumbersFinderThread.Abort();
		}

		private void MainForm_Closing(object sender,
			System.ComponentModel.CancelEventArgs e)
		{
			if (mPrimeNumbersFinderThread != null)
			{
				mPrimeNumbersFinderThread.Abort();
			}
		}
	}

	delegate void LongParameterDelegate(long aValue);

	class PrimeNumbersFinder
	{
		private MainForm mMainForm;

		public PrimeNumbersFinder(MainForm aMainForm)
		{
			mMainForm = aMainForm;
		}

		public void FindPrimeNumbers()
		{
			for (long number=0; number<1000000000; number++)
			{
				if (IsPrime(number))
				{
					mMainForm.Invoke(
						new LongParameterDelegate(mMainForm.ShowPrimeNumber), 
						new object[]{number}
					);
				}
			}
		}

		private bool IsPrime(long aNumber)
		{
			// Primarity testing. Very ineffective.
			// Don't do it in a real case!!!
			for (long i=2; i<aNumber; i++)
			{
				// Just waste some CPU time 
				int sum = 0;
				for (int w=0; w<100000; w++)
				{
					sum += w;
				}

				if (aNumber % i == 0)
				{
					return false;
				}
			}

			return true;
		}
	}
}
