using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.IO;

namespace Demo_7_MdiTextEditor
{
	/// <summary>
	/// Summary description for EditorForm.
	/// </summary>
	public class EditorForm : System.Windows.Forms.Form
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.SaveFileDialog SaveFileDialog;
		private System.Windows.Forms.RichTextBox EditorRichTextBox;

		private string mFileName = null;

		public EditorForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.SaveFileDialog = new System.Windows.Forms.SaveFileDialog();
			this.EditorRichTextBox = new System.Windows.Forms.RichTextBox();
			this.SuspendLayout();
			// 
			// SaveFileDialog
			// 
			this.SaveFileDialog.DefaultExt = "txt";
			this.SaveFileDialog.Filter = "Text files (*.txt)|*.txt";
			this.SaveFileDialog.Title = "Save File As";
			// 
			// EditorRichTextBox
			// 
			this.EditorRichTextBox.Dock = System.Windows.Forms.DockStyle.Fill;
			this.EditorRichTextBox.Location = new System.Drawing.Point(0, 0);
			this.EditorRichTextBox.Name = "EditorRichTextBox";
			this.EditorRichTextBox.Size = new System.Drawing.Size(292, 273);
			this.EditorRichTextBox.TabIndex = 1;
			this.EditorRichTextBox.Text = "";
			this.EditorRichTextBox.SelectionChanged += new System.EventHandler(this.EditorRichTextBox_SelectionChanged);
			// 
			// EditorForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(292, 273);
			this.Controls.Add(this.EditorRichTextBox);
			this.Name = "EditorForm";
			this.Text = "EditorForm";
			this.Closed += new System.EventHandler(this.EditorForm_Closed);
			this.Activated += new System.EventHandler(this.EditorForm_Activated);
			this.ResumeLayout(false);

		}
		#endregion


		public void CreateNewFile()
		{
			SetStatusBarInfo("Created new file.");
			mFileName = null;
			this.Text = "Untitled";
		}

		public void LoadFile(string aFileName)
		{
			mFileName = aFileName;
			this.Text = Path.GetFileName(aFileName);
			using (StreamReader reader = File.OpenText(aFileName))
			{
				string fileContents = reader.ReadToEnd();
				EditorRichTextBox.Text = fileContents;
			}
		}

		public void Save()
		{
			if (mFileName == null)
			{
				if (SaveFileDialog.ShowDialog() != DialogResult.OK)
				{
					return;
				}
				mFileName = SaveFileDialog.FileName;
				this.Text = Path.GetFileName(mFileName);
			}

			using (StreamWriter writer = new StreamWriter(mFileName))
			{
				writer.Write(EditorRichTextBox.Text);
			}

			SetStatusBarInfo("Saved file: " + mFileName);
		}

		private void EditorForm_Activated(object sender, System.EventArgs e)
		{
			ShowLineNumber();
		}

		private void EditorForm_Closed(object sender, System.EventArgs e)
		{
			if (mFileName != null)
			{
				SetStatusBarInfo("Closed file: " + mFileName);
			}
			else 
			{
				SetStatusBarInfo("Closed file.");
			}
			SetStatusBarLine("");
		}

		private void EditorRichTextBox_SelectionChanged(object sender, System.EventArgs e)
		{
			ShowLineNumber();
		}

		public void SetStatusBarInfo(string aText)
		{
			MainForm mainForm = (MainForm) this.MdiParent;
			mainForm.SetInfoStatusBar(aText);
		}

		public void SetStatusBarLine(string aText)
		{
			MainForm mainForm = (MainForm) this.MdiParent;
			mainForm.SetLineStatusBar(aText);
		}

		public void ShowLineNumber()
		{
			int currentPos = EditorRichTextBox.SelectionStart;
			int line = EditorRichTextBox.GetLineFromCharIndex(currentPos);
			SetStatusBarLine("Line: " + line);
		}
	}
}
